@@ -106,8 +106,8 @@ typedef struct _GailCListCellData   GailCListCellData;
 
 
 static void       gail_clist_class_init            (GailCListClass    *klass);
-static void       gail_clist_real_init             (GailWidget        *widget,
-                                                    GtkWidget         *gtk_widget);
+static void       gail_clist_real_initialize       (AtkObject         *obj,
+                                                    gpointer          data);
 static void       gail_clist_finalize              (GObject           *object);
 
 static gint       gail_clist_get_n_children        (AtkObject         *obj);
@@ -227,8 +227,7 @@ static void       gail_clist_cell_destroyed        (gpointer      data);
 static void       gail_clist_cell_data_remove      (GailCList     *clist,
                                                     GailCell      *cell);
 static GailCell*  gail_clist_find_cell             (GailCList     *clist,
-                                                    gint          row,
-                                                    gint          column);
+                                                    gint          index);
 static void       gail_clist_adjustment_changed    (GtkAdjustment *adjustment,
                                                     GtkCList      *clist);
 
@@ -315,58 +314,52 @@ gail_clist_class_init (GailCListClass *klass)
 {
   AtkObjectClass *class = ATK_OBJECT_CLASS (klass);
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-  GailWidgetClass *widget_class;
 
-  widget_class = (GailWidgetClass*)klass;
-
-  parent_class = g_type_class_ref (GAIL_TYPE_CONTAINER);
+  parent_class = g_type_class_peek_parent (klass);
  
   class->get_n_children = gail_clist_get_n_children;
   class->ref_child = gail_clist_ref_child;
-
-  widget_class->init = gail_clist_real_init;
+  class->initialize = gail_clist_real_initialize;
 
   gobject_class->finalize = gail_clist_finalize;
 }
 
-GtkAccessible* 
+AtkObject* 
 gail_clist_new (GtkWidget *widget)
 {
   GObject *object;
-  GtkAccessible *accessible;
+  AtkObject *accessible;
 
   g_return_val_if_fail (GTK_IS_CLIST (widget), NULL);
 
   object = g_object_new (GAIL_TYPE_CLIST, NULL);
 
-  g_return_val_if_fail (GTK_IS_ACCESSIBLE (object), NULL);
+  accessible = ATK_OBJECT (object);
+  atk_object_initialize (accessible, widget);
 
-  gail_widget_init (GAIL_WIDGET (object), widget);
-
-  accessible = GTK_ACCESSIBLE (object);
-  ATK_OBJECT(accessible)->role = ATK_ROLE_TABLE;
+  accessible->role = ATK_ROLE_TABLE;
 
   return accessible;
 }
 
 static void
-gail_clist_real_init (GailWidget         *widget,
-                      GtkWidget          *gtk_widget)
+gail_clist_real_initialize (AtkObject *obj,
+                            gpointer  data)
 {
   GailCList *clist;
   GtkCList *gtk_clist;
   gint i;
 
-  GAIL_WIDGET_CLASS (parent_class)->init (widget, gtk_widget);
+  ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
-  clist = GAIL_CLIST (widget);
+  clist = GAIL_CLIST (obj);
 
   clist->caption = NULL;
   clist->summary = NULL;
   clist->row_data = NULL;
   clist->cell_data = NULL;
 
-  gtk_clist = GTK_CLIST (gtk_widget);
+  gtk_clist = GTK_CLIST (data);
  
   clist->n_cols = gtk_clist->columns;
   clist->columns = g_new (GailCListColumn, gtk_clist->columns);
@@ -378,25 +371,31 @@ gail_clist_real_init (GailWidget         *widget,
   /*
    * Set up signal handlers for select-row and unselect-row
    */
-  g_signal_connect (G_OBJECT (gtk_clist),
+  g_signal_connect (gtk_clist,
                     "select-row",
                     G_CALLBACK (gail_clist_select_row_gtk),
-                    widget);
-  g_signal_connect (G_OBJECT (gtk_clist),
+                    obj);
+  g_signal_connect (gtk_clist,
                     "unselect-row",
                     G_CALLBACK (gail_clist_unselect_row_gtk),
-                    widget);
+                    obj);
   /*
    * Adjustment callbacks
    */
-  g_signal_connect (G_OBJECT (gtk_clist->hadjustment),
-                    "value_changed",
-                    G_CALLBACK (gail_clist_adjustment_changed),
-                    gtk_clist);
-  g_signal_connect (G_OBJECT (gtk_clist->vadjustment),
-                    "value_changed",
-                    G_CALLBACK (gail_clist_adjustment_changed),
-                    gtk_clist);
+  if (gtk_clist->hadjustment)
+    {
+      g_signal_connect (gtk_clist->hadjustment,
+                        "value_changed",
+                        G_CALLBACK (gail_clist_adjustment_changed),
+                        gtk_clist);
+    }
+  if (gtk_clist->vadjustment)
+    {
+      g_signal_connect (gtk_clist->vadjustment,
+                        "value_changed",
+                        G_CALLBACK (gail_clist_adjustment_changed),
+                        gtk_clist);
+    }
 }
 
 static void
@@ -656,6 +655,7 @@ gail_clist_ref_at_actual (AtkTable      *table,
   GtkCellType cellType;
   AtkObject *return_object;
   gint n_rows, n_columns;
+  gint index;
   GailCell *cell;
 
   g_return_val_if_fail (GTK_IS_ACCESSIBLE (table), NULL);
@@ -677,7 +677,8 @@ gail_clist_ref_at_actual (AtkTable      *table,
   /*
    * Check whether the child is cached
    */
-  cell = gail_clist_find_cell (GAIL_CLIST (table), row, column);
+  index =  column + row * n_columns;
+  cell = gail_clist_find_cell (GAIL_CLIST (table), index);
   if (cell)
     {
       g_object_ref (cell);
@@ -707,8 +708,7 @@ gail_clist_ref_at_actual (AtkTable      *table,
       g_return_val_if_fail (ATK_IS_OBJECT (table), NULL);
 
       gail_cell_init (cell, widget, ATK_OBJECT (table),
-                      row, column,
-                      column + row * gail_clist_get_n_actual_columns (clist));
+                      index);
       /*
        * Store the cell in a cache
        */
@@ -814,6 +814,9 @@ gail_clist_get_column_header (AtkTable      *table,
   if (return_widget == NULL)
     return NULL;
 
+  g_return_val_if_fail (GTK_IS_BIN (return_widget), NULL);
+  return_widget = gtk_bin_get_child (GTK_BIN(return_widget));
+
   return gtk_widget_get_accessible (return_widget);
 }
 
@@ -1157,15 +1160,20 @@ gail_clist_get_cell_area (GailCellParent *parent,
 {
   GtkWidget* widget;
   GtkCList *clist;
+  gint column, row, n_columns;
 
   widget = GTK_ACCESSIBLE (parent)->widget;
   if (widget == NULL)
     return;
   clist = GTK_CLIST (widget);
 
-  cell_rect->x = COLUMN_LEFT (clist, cell->column);
-  cell_rect->y = ROW_TOP (clist, cell->row);
-  cell_rect->width = clist->column[cell->column].area.width;
+  n_columns = gail_clist_get_n_actual_columns (clist);
+  g_return_if_fail (n_columns > 0);
+  column = cell->index % n_columns;
+  row = cell->index / n_columns; 
+  cell_rect->x = COLUMN_LEFT (clist, column);
+  cell_rect->y = ROW_TOP (clist, row);
+  cell_rect->width = clist->column[column].area.width;
   cell_rect->height = clist->row_height;
 }
 
@@ -1335,7 +1343,7 @@ gail_clist_set_row_data (AtkTable      *table,
           break;
         }
     } 
-  if (found == FALSE)
+  if (!found)
     {
       row_data = g_new (GailCListRow, 1);
       row_data->row_number = row;
@@ -1496,19 +1504,22 @@ gail_clist_cell_data_remove (GailCList *clist,
 
 static GailCell*
 gail_clist_find_cell (GailCList     *clist,
-                      gint          row,
-                      gint          column)
+                      gint          index)
 {
   GList *temp_list;
+  gint n_cols;
+
+  n_cols = clist->n_cols;
 
   for (temp_list = clist->cell_data; temp_list; temp_list = temp_list->next)
     {
       GailCListCellData *cell_data;
+      gint real_index;
 
       cell_data = (GailCListCellData *) (temp_list->data);
 
-      if ((cell_data->column_number == column) &&
-          (cell_data->row_number == row))
+      real_index = cell_data->column_number + n_cols * cell_data->row_number;
+      if (real_index == index)
         return cell_data->gail_cell;
     }
   return NULL;
