@@ -22,7 +22,7 @@
 
 #include <gtk/gtkaccessible.h>
 #include <gail/gailwidget.h>
-#include <gail/gailtexthelper.h>
+#include <util/gailtextutil.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -42,7 +42,7 @@ struct _GailEntry
 {
   GailWidget parent;
 
-  GailTextHelper *texthelper;
+  GailTextUtil *textutil;
   /*
    * These fields store information about text changed
    */
