@@ -17,14 +17,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <string.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include "gailoptionmenu.h"
 
 static void                  gail_option_menu_class_init       (GailOptionMenuClass *klass);
-static gint                  gail_option_menu_get_n_children   (AtkObject       *obj);
-static AtkObject*            gail_option_menu_ref_child        (AtkObject       *obj,
-                                                                gint            i);
 
 static void                  atk_action_interface_init         (AtkActionIface  *iface);
 
@@ -47,33 +45,33 @@ gail_option_menu_get_type (void)
   static GType type = 0;
 
   if (!type)
-  {
-    static const GTypeInfo tinfo =
-    {
-      sizeof (GailOptionMenuClass),
-      (GBaseInitFunc) NULL, /* base init */
-      (GBaseFinalizeFunc) NULL, /* base finalize */
-      (GClassInitFunc) gail_option_menu_class_init, /* class init */
-      (GClassFinalizeFunc) NULL, /* class finalize */
-      NULL, /* class data */
-      sizeof (GailOptionMenu), /* instance size */
-      0, /* nb preallocs */
-      (GInstanceInitFunc) NULL, /* instance init */
-      NULL /* value table */
-    };
-
-    static const GInterfaceInfo atk_action_info =
     {
+      static const GTypeInfo tinfo =
+      {
+        sizeof (GailOptionMenuClass),
+        (GBaseInitFunc) NULL, /* base init */
+        (GBaseFinalizeFunc) NULL, /* base finalize */
+        (GClassInitFunc) gail_option_menu_class_init, /* class init */
+        (GClassFinalizeFunc) NULL, /* class finalize */
+        NULL, /* class data */
+        sizeof (GailOptionMenu), /* instance size */
+        0, /* nb preallocs */
+        (GInstanceInitFunc) NULL, /* instance init */
+        NULL /* value table */
+      };
+  
+      static const GInterfaceInfo atk_action_info =
+      {
         (GInterfaceInitFunc) atk_action_interface_init,
         (GInterfaceFinalizeFunc) NULL,
         NULL
-    };
+      };
 
-    type = g_type_register_static (GAIL_TYPE_BUTTON,
-                                   "GailOptionMenu", &tinfo, 0);
-    g_type_add_interface_static (type, ATK_TYPE_ACTION,
-                                 &atk_action_info);
-  }
+      type = g_type_register_static (GAIL_TYPE_BUTTON,
+                                     "GailOptionMenu", &tinfo, 0);
+      g_type_add_interface_static (type, ATK_TYPE_ACTION,
+                                   &atk_action_info);
+    }
 
   return type;
 }
@@ -81,35 +79,23 @@ gail_option_menu_get_type (void)
 static void
 gail_option_menu_class_init (GailOptionMenuClass *klass)
 {
-  AtkObjectClass *class = ATK_OBJECT_CLASS (klass);
-  GailButtonClass *button_class = GAIL_BUTTON_CLASS (klass);
-
-  parent_class = g_type_class_ref (GAIL_TYPE_BUTTON);
-
-  class->get_n_children = gail_option_menu_get_n_children;
-  class->ref_child = gail_option_menu_ref_child;
+  parent_class = g_type_class_peek_parent (klass);
 }
 
-GtkAccessible* 
+AtkObject* 
 gail_option_menu_new (GtkWidget *widget)
 {
   GObject *object;
-  GtkAccessible *accessible;
-  AtkObject *child_accessible;
+  AtkObject *accessible;
 
   g_return_val_if_fail (GTK_IS_OPTION_MENU (widget), NULL);
 
   object = g_object_new (GAIL_TYPE_OPTION_MENU, NULL);
 
-  g_return_val_if_fail (GTK_IS_ACCESSIBLE (object), NULL);
+  accessible = ATK_OBJECT (object);
+  atk_object_initialize (accessible, widget);
 
-  gail_widget_init (GAIL_WIDGET (object), widget);
-
-  accessible = GTK_ACCESSIBLE (object);
-  ATK_OBJECT(accessible)->role = ATK_ROLE_PUSH_BUTTON;
-
-  child_accessible = gtk_widget_get_accessible (GTK_OPTION_MENU (widget)->menu);
-  atk_object_set_parent (child_accessible, ATK_OBJECT (accessible));
+  accessible->role = ATK_ROLE_PUSH_BUTTON;
 
   return accessible;
 }
@@ -122,7 +108,6 @@ atk_action_interface_init (AtkActionIface *iface)
   iface->do_action = gail_option_menu_do_action;
   iface->get_n_actions = gail_option_menu_get_n_actions;
   iface->get_description = gail_option_menu_get_description;
-  iface->get_keybinding = NULL;
   iface->get_name = gail_option_menu_action_get_name;
   iface->set_description = gail_option_menu_set_description;
 }
@@ -137,40 +122,38 @@ gail_option_menu_do_action (AtkAction *action,
 
   widget = GTK_ACCESSIBLE (action)->widget;
   if (widget == NULL)
-  {
     /*
      * State is defunct
      */
     return FALSE;
-  }
 
   button = GTK_BUTTON (widget); 
   switch (i)
-  {
-    case 0:
     {
-      GdkEvent tmp_event;
+    case 0:
+      {
+        GdkEvent tmp_event;
 
-      button->in_button = TRUE;
-      gtk_button_enter (button);
-      /*
-       * Simulate a button press event. calling gtk_button_pressed() does
-       * not get the job done for a GtkOptionMenu.  
-       */
-      tmp_event.button.type = GDK_BUTTON_PRESS;
-      tmp_event.button.window = widget->window;
-      tmp_event.button.button = 1;
-      tmp_event.button.send_event = TRUE;
-      tmp_event.button.time = GDK_CURRENT_TIME;
-      tmp_event.button.axes = NULL;
+        button->in_button = TRUE;
+        gtk_button_enter (button);
+        /*
+         * Simulate a button press event. calling gtk_button_pressed() does
+         * not get the job done for a GtkOptionMenu.  
+         */
+        tmp_event.button.type = GDK_BUTTON_PRESS;
+        tmp_event.button.window = widget->window;
+        tmp_event.button.button = 1;
+        tmp_event.button.send_event = TRUE;
+        tmp_event.button.time = GDK_CURRENT_TIME;
+        tmp_event.button.axes = NULL;
 
-      gtk_widget_event (widget, &tmp_event);
-      break;
-    }
+        gtk_widget_event (widget, &tmp_event);
+        break;
+      }
     default:
       return_value = FALSE;
       break;
-  }
+    }
   return return_value; 
 }
 
@@ -190,14 +173,14 @@ gail_option_menu_get_description (AtkAction *action,
   button = GAIL_BUTTON (action);
 
   switch (i)
-  {
+    {
     case 0:
       return_value = button->press_description;
       break;
     default:
       return_value = NULL;
       break;
-  }
+    }
   return return_value; 
 }
 
@@ -208,7 +191,7 @@ gail_option_menu_action_get_name (AtkAction *action,
   G_CONST_RETURN gchar *return_value;
 
   switch (i)
-  {
+    {
     case 0:
       /*
        * This action simulates a button press by simulating moving the
@@ -234,73 +217,21 @@ gail_option_menu_set_description (AtkAction      *action,
   button = GAIL_BUTTON (action);
 
   switch (i)
-  {
+    {
     case 0:
       value = &button->press_description;
       break;
     default:
       value = NULL;
       break;
-  }
+    }
+
   if (value)
-  {
-    g_free (*value);
-    *value = g_strdup (desc);
-    return TRUE;
-  }
+    {
+      g_free (*value);
+      *value = g_strdup (desc);
+      return TRUE;
+    }
   else
-  {
     return FALSE;
-  }
-}
-
-static gint
-gail_option_menu_get_n_children (AtkObject* obj)
-{
-  GtkOptionMenu *option_menu; 
-  GtkWidget *widget;
-
-  widget = GTK_ACCESSIBLE (obj)->widget;
-  if (widget == NULL)
-  {
-    /*
-     * State is defunct
-     */
-    return 0;
-  }
-
-  option_menu = GTK_OPTION_MENU (widget); 
-  if (option_menu->menu)
-    return 1;
-  else 
-    return 0;
-}
-
-static AtkObject*
-gail_option_menu_ref_child (AtkObject *obj,
-                            gint      i)
-{
-  GtkOptionMenu *option_menu; 
-  GtkWidget *widget;
-
-  widget = GTK_ACCESSIBLE (obj)->widget;
-  if (widget == NULL)
-  {
-    /*
-     * State is defunct
-     */
-    return NULL;
-  }
-
-  option_menu = GTK_OPTION_MENU (widget); 
-  if (option_menu->menu)
-  {
-    AtkObject *child;
-
-    child = gtk_widget_get_accessible (option_menu->menu);
-    g_object_ref (child);
-    return child;
-  }
-  else 
-    return NULL;
 }
