@@ -17,14 +17,16 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <string.h>
 #include <gtk/gtk.h>
 #include "gailscrolledwindow.h"
 #include "gailscrolledwindowfactory.h"
 
 
 static void         gail_scrolled_window_class_init     (GailScrolledWindowClass  *klass); 
-static void         gail_scrolled_window_real_init      (GailWidget    *widget,
-                                                         GtkWidget     *gtk_widget);
+static void         gail_scrolled_window_real_initialize
+                                                        (AtkObject     *obj,
+                                                         gpointer      data);
 
 static gint         gail_scrolled_window_get_n_children (AtkObject     *object);
 static AtkObject*   gail_scrolled_window_ref_child      (AtkObject     *obj,
@@ -66,52 +68,48 @@ gail_scrolled_window_get_type (void)
 static void
 gail_scrolled_window_class_init (GailScrolledWindowClass *klass)
 {
-  GailWidgetClass *widget_class;
   AtkObjectClass  *class = ATK_OBJECT_CLASS (klass);
 
-  widget_class = (GailWidgetClass*)klass;
-  widget_class->init = gail_scrolled_window_real_init;
+  parent_class = g_type_class_peek_parent (klass);
 
-  parent_class = g_type_class_ref (GAIL_TYPE_CONTAINER);
   class->get_n_children = gail_scrolled_window_get_n_children;
   class->ref_child = gail_scrolled_window_ref_child;
+  class->initialize = gail_scrolled_window_real_initialize;
 }
 
-GtkAccessible* 
+AtkObject* 
 gail_scrolled_window_new (GtkWidget *widget)
 {
   GObject *object;
-  GtkAccessible *accessible;
+  AtkObject *accessible;
   
   g_return_val_if_fail (GTK_IS_SCROLLED_WINDOW (widget), NULL);
 
   object = g_object_new (GAIL_TYPE_SCROLLED_WINDOW, NULL);
 
-  g_return_val_if_fail (GTK_IS_ACCESSIBLE (object), NULL);
+  accessible = ATK_OBJECT (object);
+  atk_object_initialize (accessible, widget);
 
-  gail_widget_init (GAIL_WIDGET (object), widget);
-
-  accessible = GTK_ACCESSIBLE (object);
-  ATK_OBJECT(accessible)->role = ATK_ROLE_SCROLL_PANE;
+  accessible->role = ATK_ROLE_SCROLL_PANE;
 
   return accessible;
 }
 
 static void
-gail_scrolled_window_real_init (GailWidget         *widget,
-                                GtkWidget          *gtk_widget)
+gail_scrolled_window_real_initialize (AtkObject *obj,
+                                      gpointer  data)
 {
   GtkScrolledWindow *window;
 
-  GAIL_WIDGET_CLASS (parent_class)->init (widget, gtk_widget);
+  ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
-  window = GTK_SCROLLED_WINDOW (gtk_widget);
-  g_signal_connect_data (G_OBJECT (window->hscrollbar), "notify::visible",
+  window = GTK_SCROLLED_WINDOW (data);
+  g_signal_connect_data (window->hscrollbar, "notify::visible",
     (GCallback)gail_scrolled_window_scrollbar_visibility_changed, 
-    widget, FALSE, FALSE);
-  g_signal_connect_data (G_OBJECT (window->vscrollbar), "notify::visible",
+    obj, FALSE, FALSE);
+  g_signal_connect_data (window->vscrollbar, "notify::visible",
     (GCallback)gail_scrolled_window_scrollbar_visibility_changed, 
-    widget, FALSE, FALSE);
+    obj, FALSE, FALSE);
 }
 
 static gint
@@ -154,7 +152,7 @@ gail_scrolled_window_ref_child (AtkObject *obj,
   gint n_children;
   AtkObject  *accessible = NULL;
 
-  g_return_val_if_fail ((child >= 0), NULL);
+  g_return_val_if_fail (child >= 0, NULL);
 
   widget = GTK_ACCESSIBLE (obj)->widget;
   if (widget == NULL)
@@ -200,6 +198,7 @@ gail_scrolled_window_scrollbar_visibility_changed (GObject    *object,
       AtkObject *child;
       GtkScrolledWindow *gtk_window;
       GailScrolledWindow *gail_window = GAIL_SCROLLED_WINDOW (user_data);
+      gchar *signal_name;
 
       gtk_window = GTK_SCROLLED_WINDOW (GTK_ACCESSIBLE (user_data)->widget);
       children = gtk_container_children (GTK_CONTAINER (gtk_window));
@@ -208,27 +207,30 @@ gail_scrolled_window_scrollbar_visibility_changed (GObject    *object,
       if ((gpointer) object == (gpointer) (gtk_window->hscrollbar))
         {
           if (gtk_window->hscrollbar_visible)
-            {
-              child_added = TRUE;
-              child = gtk_widget_get_accessible (gtk_window->hscrollbar);
-            }
+            child_added = TRUE;
+
+          child = gtk_widget_get_accessible (gtk_window->hscrollbar);
         }
       else if ((gpointer) object == (gpointer) (gtk_window->vscrollbar))
         {
           if (gtk_window->vscrollbar_visible)
-            {
-              child_added = TRUE;
-              child = gtk_widget_get_accessible (gtk_window->vscrollbar);
-            }
+            child_added = TRUE;
+
+          child = gtk_widget_get_accessible (gtk_window->vscrollbar);
           if (gtk_window->hscrollbar_visible)
             index = n_children+1;
         }
+      else 
+        {
+          g_assert_not_reached ();
+          return;
+        }
 
       if (child_added)
-        g_signal_emit_by_name (gail_window, "children_changed::add",
-                               index, child, NULL);
+        signal_name = "children_changed::add";
       else
-        g_signal_emit_by_name (gail_window, "children_changed::delete",
-                               index, child, NULL);
+        signal_name = "children_changed::delete";
+
+      g_signal_emit_by_name (gail_window, signal_name, index, child, NULL);
     }
 }
