@@ -27,22 +27,27 @@ static void		gail_util_class_init			(GailUtilClass		*klass);
 /* atkutil.h */
 
 static guint		gail_util_add_global_event_listener	(GSignalEmissionHook	listener,
-								gchar*			event_type);
+							         const gchar*           event_type);
 static void 		gail_util_remove_global_event_listener	(guint			remove_listener);
+static guint		gail_util_add_key_event_listener	(AtkKeySnoopFunc	listener,
+								 gpointer               data);
+static void 		gail_util_remove_key_event_listener	(guint			remove_listener);
 static AtkObject*	gail_util_get_root			(void);
-static gchar*		gail_util_get_toolkit_name		(void);
-static gchar* 		gail_util_get_toolkit_version		(void);
+static G_CONST_RETURN gchar *gail_util_get_toolkit_name		(void);
+static G_CONST_RETURN gchar *gail_util_get_toolkit_version      (void);
 
 /* Misc */
 
 static void		_listener_info_destroy			(gpointer		data);
 
 static AtkObject* root = NULL;
-static gpointer parent_class = NULL;
 static GHashTable *listener_list = NULL;
 static gint listener_idx = 1;
+static GHashTable *key_listener_list = NULL;
+static guint key_snooper_id = 0;
 
 typedef struct _GailUtilListenerInfo GailUtilListenerInfo;
+typedef struct _GailKeyEventInfo GailKeyEventInfo;
 
 struct _GailUtilListenerInfo
 {
@@ -51,6 +56,12 @@ struct _GailUtilListenerInfo
    gulong hook_id;
 };
 
+struct _GailKeyEventInfo
+{
+  AtkKeyEventStruct *key_event;
+  gpointer func_data;
+};
+
 GType
 gail_util_get_type (void)
 {
@@ -84,8 +95,6 @@ gail_util_class_init (GailUtilClass *klass)
   AtkUtilClass *atk_class;
   gpointer data;
 
-  parent_class = g_type_class_ref (ATK_TYPE_UTIL);
-
   data = g_type_class_peek (ATK_TYPE_UTIL);
   atk_class = ATK_UTIL_CLASS (data);
 
@@ -93,6 +102,10 @@ gail_util_class_init (GailUtilClass *klass)
     gail_util_add_global_event_listener;
   atk_class->remove_global_event_listener =
     gail_util_remove_global_event_listener;
+  atk_class->add_key_event_listener =
+    gail_util_add_key_event_listener;
+  atk_class->remove_key_event_listener =
+    gail_util_remove_key_event_listener;
   atk_class->get_root = gail_util_get_root;
   atk_class->get_toolkit_name = gail_util_get_toolkit_name;
   atk_class->get_toolkit_version = gail_util_get_toolkit_version;
@@ -103,7 +116,7 @@ gail_util_class_init (GailUtilClass *klass)
 
 static guint
 gail_util_add_global_event_listener (GSignalEmissionHook listener,
-   gchar* event_type)
+				     const gchar *event_type)
 {
   GType type;
   guint signal_id;
@@ -122,11 +135,12 @@ gail_util_add_global_event_listener (GSignalEmissionHook listener,
 
           rc = listener_idx;
 
-          listener_info = malloc(sizeof(GailUtilListenerInfo));
+          listener_info = g_malloc(sizeof(GailUtilListenerInfo));
           listener_info->key = listener_idx;
           listener_info->hook_id =
-            g_signal_add_emission_hook (signal_id, 0, listener,
-            event_type, (GDestroyNotify) NULL);
+		  g_signal_add_emission_hook (signal_id, 0, listener,
+					      g_strdup (event_type),
+					      (GDestroyNotify) g_free);
           listener_info->signal_id = signal_id;
 
 	  g_hash_table_insert(listener_list, &(listener_info->key),
@@ -190,6 +204,94 @@ gail_util_remove_global_event_listener (guint remove_listener)
   }
 }
 
+
+static
+AtkKeyEventStruct *
+atk_key_event_from_gdk_event_key (GdkEventKey *key)
+{
+  AtkKeyEventStruct *event = g_new0 (AtkKeyEventStruct, 1);
+  switch (key->type)
+    {
+    case GDK_KEY_PRESS:
+	    event->type = ATK_KEY_EVENT_PRESS;
+	    break;
+    case GDK_KEY_RELEASE:
+	    event->type = ATK_KEY_EVENT_RELEASE;
+	    break;
+    default:
+    }
+  event->state = key->state;
+  event->keyval = key->keyval;
+  event->length = key->length;
+  if (key->string && key->string [0] && g_unichar_isgraph (g_utf8_get_char (key->string)))
+    {
+      event->string = key->string;
+    }
+  else if (key->type == GDK_KEY_PRESS)
+    {
+      event->string = gdk_keyval_name (key->keyval);	    
+    }
+  event->keycode = key->hardware_keycode;
+  event->timestamp = key->time;
+#ifdef GAIL_DEBUG  
+  g_print ("GailKey:\tsym %u\n\tmods %x\n\tcode %u\n\ttime %lx\n",
+	   (unsigned int) event->keyval,
+	   (unsigned int) event->state,
+	   (unsigned int) event->keycode,
+	   (unsigned long int) event->timestamp);
+#endif
+  return event;
+}
+
+static void
+notify_hf (gpointer key, gpointer value, gpointer data)
+{
+  GailKeyEventInfo *info = (GailKeyEventInfo *) data;
+  (*(AtkKeySnoopFunc) value) (info->key_event, info->func_data);
+}
+
+static gint
+gail_key_snooper (GtkWidget *the_widget, GdkEventKey *event, gpointer func_data)
+{
+  /* notify each AtkKeySnoopFunc in turn... */
+  GailKeyEventInfo *info = g_new0 (GailKeyEventInfo, 1);
+  if (key_listener_list)
+    {
+      info->key_event = atk_key_event_from_gdk_event_key (event);
+      info->func_data = func_data;
+      g_hash_table_foreach (key_listener_list, notify_hf, info);
+    }
+  g_free (info->key_event);
+  g_free (info);
+  return 0;
+}
+
+static guint
+gail_util_add_key_event_listener (AtkKeySnoopFunc  listener,
+				  gpointer         data)
+{
+  static guint key=0;
+  if (!key_listener_list)
+  {
+    key_listener_list = g_hash_table_new (NULL, NULL);	  
+    key_snooper_id = gtk_key_snooper_install (gail_key_snooper, data);
+  }
+  /* XXX: slightly fishy use of int as a 'pointer', is this OK? */
+  g_hash_table_insert (key_listener_list, (gpointer) key++, listener);
+  /* XXX: we don't check to see if n_listeners > MAXUINT */
+  return key;
+}
+
+static void
+gail_util_remove_key_event_listener (guint remove_listener)
+{
+  g_hash_table_remove (key_listener_list, GUINT_TO_POINTER (remove_listener));
+  if (g_hash_table_size (key_listener_list) == 0)
+    {
+      gtk_key_snooper_remove (key_snooper_id);
+    }
+}
+
 static AtkObject*
 gail_util_get_root (void)
 {
@@ -199,20 +301,20 @@ gail_util_get_root (void)
   return root;
 }
 
-static gchar*
+static G_CONST_RETURN gchar *
 gail_util_get_toolkit_name (void)
 {
-  return ("GAIL");
+  return "GAIL";
 }
 
-static gchar*
+static G_CONST_RETURN gchar *
 gail_util_get_toolkit_version (void)
 {
  /*
   * Version is passed in as a -D flag when this file is
   * compiled.
   */
-  return (VERSION);
+  return VERSION;
 }
 
 static void
