@@ -52,6 +52,8 @@ static void     gail_widget_get_size       (AtkComponent    *component,
                                             gint            *width,
                                             gint            *height);
 
+static AtkLayer gail_widget_get_layer      (AtkComponent *component);
+
 static gboolean gail_widget_grab_focus     (AtkComponent    *component);
 
 
@@ -175,7 +177,6 @@ gail_widget_real_initialize (AtkObject *obj,
   g_return_if_fail (GTK_IS_WIDGET (data));
 
   widget = GTK_WIDGET (data);
-  obj->layer = ATK_LAYER_WIDGET;
 
   accessible = GTK_ACCESSIBLE (obj);
   accessible->widget = widget;
@@ -221,6 +222,8 @@ gail_widget_new (GtkWidget *widget)
   atk_object_initialize (accessible, widget);
 
   accessible->role = ATK_ROLE_UNKNOWN;
+  g_object_set_data (G_OBJECT (accessible), "atk-component-layer",
+		     GINT_TO_POINTER (ATK_LAYER_WIDGET));
 
   return accessible;
 }
@@ -250,6 +253,7 @@ static void
 gail_widget_destroyed (GtkWidget     *widget,
                        GtkAccessible *accessible)
 {
+  accessible->widget = NULL;
   atk_object_notify_state_change (ATK_OBJECT (accessible), ATK_STATE_DEFUNCT,
                                   TRUE); 
 }
@@ -326,6 +330,7 @@ gail_widget_get_parent (AtkObject *accessible)
          * State is defunct
          */
         return NULL;
+      g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
 
       parent_widget = widget->parent;
       if (parent_widget == NULL)
@@ -446,7 +451,10 @@ gail_widget_get_index_in_parent (AtkObject *accessible)
         }
     }
 
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), -1);
   parent_widget = widget->parent;
+  if (parent_widget == NULL)
+    return -1;
   g_return_val_if_fail (GTK_IS_CONTAINER (parent_widget), -1);
 
   children = gtk_container_children (GTK_CONTAINER (parent_widget));
@@ -467,6 +475,7 @@ atk_component_interface_init (AtkComponentIface *iface)
   iface->add_focus_handler = gail_widget_add_focus_handler;
   iface->get_extents = gail_widget_get_extents;
   iface->get_size = gail_widget_get_size;
+  iface->get_layer = gail_widget_get_layer;
   iface->grab_focus = gail_widget_grab_focus;
   iface->remove_focus_handler = gail_widget_remove_focus_handler;
   iface->set_extents = gail_widget_set_extents;
@@ -521,6 +530,8 @@ gail_widget_get_extents (AtkComponent   *component,
      */
     return;
 
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+
   *width = widget->allocation.width;
   *height = widget->allocation.height;
   if (!gail_widget_on_screen (widget) || (!GTK_WIDGET_DRAWABLE (widget)))
@@ -570,16 +581,29 @@ gail_widget_get_size (AtkComponent   *component,
      */
     return;
 
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+
   *width = widget->allocation.width;
   *height = widget->allocation.height;
 }
 
+static AtkLayer
+gail_widget_get_layer (AtkComponent *component)
+{
+  gint layer;
+  layer = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (component), "atk-component-layer"));
+  g_assert (layer >= ATK_LAYER_INVALID);
+  g_assert (layer <= ATK_LAYER_OVERLAY);
+
+  return (AtkLayer) layer;
+}
+
 static gboolean 
 gail_widget_grab_focus (AtkComponent   *component)
 {
   GtkWidget *widget = GTK_ACCESSIBLE (component)->widget;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
   if (GTK_WIDGET_CAN_FOCUS (widget))
     {
       gtk_widget_grab_focus (widget);
@@ -611,6 +635,7 @@ gail_widget_set_extents (AtkComponent   *component,
      * Object is defunct
      */
     return FALSE;
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
 
   if (GTK_WIDGET_TOPLEVEL (widget))
     {
@@ -654,6 +679,7 @@ gail_widget_set_position (AtkComponent   *component,
      * Object is defunct
      */
     return FALSE;
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
 
   if (GTK_WIDGET_TOPLEVEL (widget))
     {
@@ -694,6 +720,7 @@ gail_widget_set_size (AtkComponent   *component,
      * Object is defunct
      */
     return FALSE;
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
 
   if (GTK_WIDGET_TOPLEVEL (widget))
     {
