@@ -176,7 +176,14 @@ gail_window_get_name (AtkObject *accessible)
 static AtkObject*
 gail_window_get_parent (AtkObject *accessible)
 {
-  return atk_get_root ();
+  AtkObject* parent;
+
+  parent = ATK_OBJECT_CLASS (parent_class)->get_parent (accessible);
+
+  if (!parent)
+    parent = atk_get_root ();
+
+  return parent;
 }
 
 static gint
@@ -185,7 +192,7 @@ gail_window_get_index_in_parent (AtkObject *accessible)
   GtkWidget* widget = GTK_ACCESSIBLE (accessible)->widget; 
   AtkObject* atk_obj = atk_get_root ();
   GailToplevel* toplevel = GAIL_TOPLEVEL (atk_obj);
-  guint window_count = -1;
+  gint index = -1;
 
   if (widget == NULL)
     /*
@@ -195,13 +202,17 @@ gail_window_get_index_in_parent (AtkObject *accessible)
 
   g_return_val_if_fail (GTK_IS_WIDGET (widget), -1);
 
+  index = ATK_OBJECT_CLASS (parent_class)->get_index_in_parent (accessible);
+  if (index != -1)
+    return index;
+
   if (GTK_IS_WINDOW (widget))
     {
       GtkWindow *window = GTK_WINDOW (widget);
 
-      window_count = g_list_index (toplevel->window_list, window);
+      index = g_list_index (toplevel->window_list, window);
     }
-  return window_count;
+  return index;
 }
 
 static gboolean
