@@ -4,12 +4,12 @@
 #define MAX_NAME_VALUE 20
 
 #include <netinet/in.h>
-#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <errno.h>
 #include <unistd.h>
+#include <stdlib.h>
 #include <strings.h>
 #include "testlib.h"
 
@@ -24,7 +24,7 @@ typedef enum
   TEXT,
   VALUE,
   END_TABS
-}TabNumber;
+} TabNumber;
 
 typedef enum
 {
@@ -39,7 +39,7 @@ typedef enum
   TEXT_INTERFACE,
   TEXT_ATTRIBUTES,
   VALUE_INTERFACE
-}GroupId;
+} GroupId;
 
 typedef enum
 {
@@ -47,7 +47,7 @@ typedef enum
   VALUE_BOOLEAN,
   VALUE_TEXT,
   VALUE_BUTTON
-}ValueType;
+} ValueType;
 
 /* GUI Information for the group */
 
@@ -62,7 +62,7 @@ typedef struct
   gchar         *name;
   gboolean      is_scrolled;
   gint          default_height;
-}GroupInfo;
+} GroupInfo;
 
 typedef struct
 {
@@ -70,7 +70,7 @@ typedef struct
   GtkWidget *page;
   GtkWidget *main_box;
   gchar     *name;
-}TabInfo;
+} TabInfo;
 
 typedef struct
 {
@@ -89,7 +89,7 @@ typedef struct
   GtkWidget *string;
   GtkWidget *boolean;
   GtkWidget *text;
-}NameValue;
+} NameValue;
 
 typedef enum {
    FERRET_SIGNAL_OBJECT,
@@ -165,9 +165,9 @@ static NameValue* _get_name_value(GroupInfo *group, const gchar *label,
 
 static void _update_handlers(AtkObject *obj);
 static void _notify_text_insert_handler (GObject *obj,
-  int start_offset, int end_offset);
+  int position, int offset);
 static void _notify_text_delete_handler (GObject *obj,
-  int start_offset, int end_offset);
+  int position, int offset);
 static void _notify_caret_handler (GObject *obj, int position);
 static void _notify_table_row_inserted (GObject *obj,
   gint start_offset, gint length);
@@ -183,6 +183,8 @@ static void _notify_object_child_added (GObject *obj,
   gint index, AtkObject *child);
 static void _notify_object_child_removed (GObject *obj,
   gint index, AtkObject *child);
+static void _notify_object_state_change (GObject *obj,
+  gchar *name, gboolean set);
 
 /* Property handlers */
 
@@ -204,15 +206,19 @@ void _toggle_festival_terse(GtkCheckMenuItem *checkmenuitem,
   gpointer user_data);
 void _toggle_trackmouse(GtkCheckMenuItem *checkmenuitem,
   gpointer user_data);
+void _toggle_trackfocus(GtkCheckMenuItem *checkmenuitem,
+  gpointer user_data);
 
 /* Global variables */
 static GtkNotebook *notebook;
 static TabInfo  *nbook_tabs[END_TABS];
-static gboolean mouse_watcher_focus_id = -1;
-static gboolean mouse_watcher_button_id = -1;
+static gint mouse_watcher_focus_id = -1;
+static gint mouse_watcher_button_id = -1;
+static gint focus_tracker_id = -1;
 static gboolean use_magnifier = FALSE;
 static gboolean use_festival = FALSE;
 static gboolean track_mouse = FALSE;
+static gboolean track_focus = TRUE;
 static gboolean say_role = TRUE;
 static gboolean say_accel = TRUE;
 static gboolean display_ascii = FALSE;
@@ -231,6 +237,7 @@ static GtkWidget *menuitem_magnifier = NULL;
 static GtkWidget *menuitem_festival = NULL;
 static GtkWidget *menuitem_festival_terse = NULL;
 static GtkWidget *menuitem_trackmouse = NULL;
+static GtkWidget *menuitem_trackfocus = NULL;
 
 static struct sockaddr_un mag_server = { AF_UNIX , "/tmp/magnifier_socket" };
 static struct sockaddr_un client = { AF_UNIX, "/tmp/mag_client"};
@@ -308,16 +315,24 @@ static void _festival_say (const gchar *text)
   static int fd = 0;
   gchar *quoted;
   gchar *p;
+  gchar prefix [100];
+  const gchar *stretch;
+
+  fprintf (stderr, "saying %s\n", text);
 
   if (!fd)
     {
       fd = _festival_init ();
     }
 
-  quoted = g_malloc(64+strlen(text)*2);
+  quoted = g_malloc(100+strlen(text)*2);
 
-  strcpy(quoted, "(audio_mode'shutup)\n (SayText \"");
-  p = quoted+31;
+  stretch = g_strdup (g_getenv ("FESTIVAL_STRETCH"));
+  if (!stretch) stretch = "0.75";
+  sprintf (prefix, "(audio_mode'shutup)\n (Parameter.set 'Duration_Stretch %s)\n (SayText \"", stretch);
+  
+  strcpy(quoted, prefix);
+  p = quoted + strlen (prefix);
   while (*text) {
     if ( *text == '\\' || *text == '"' )
       *p = '\\';
@@ -337,17 +352,17 @@ static void _send_to_festival (const gchar *role_name,
   const gchar *name, const gchar *accel)
 {
   gchar *string;
-  int len = (strlen (role_name)+1 - 9 + strlen (name)+2 + 4 + strlen (accel)+2);
-                   /* omit " atk_role_" and add 2 additional spaces */
+  int len = (strlen (role_name)+1 + strlen (name)+2 + 4 + strlen (accel)+2);
   int i, j;
   gchar ch;
   gchar *accel_name;
+  
   string = (gchar *) g_malloc (len * sizeof (gchar));
 
   i = 0;
   if (say_role)
     {
-      j = 9;
+      j = 0;
       while (role_name[j])
         {
           ch = role_name[j++];
@@ -360,10 +375,8 @@ static void _send_to_festival (const gchar *role_name,
   while (name[j])
     {
       ch = name[j++];
-      if (ch != '_')
-        {
-          string[i++] = ch;
-        }
+      if (ch == '_') ch = ' ';
+      string[i++] = ch;
     };
   if ((say_accel) && (strlen (accel) > 0))
     {
@@ -389,6 +402,7 @@ static void _send_to_festival (const gchar *role_name,
         };
     }
   string[i] = '\0';
+
   _festival_say (string);
   g_free (string);
 }
@@ -471,7 +485,7 @@ static void _print_accessible (AtkObject *aobject)
     g_print("\nFocus change\n");
 
   /* Do not attach signal handlers if the user has asked not to */
-  if (no_signals == FALSE)
+  if (!no_signals)
     _update_handlers(aobject);
   else
     last_object = aobject; /* _update_handler normally does this */
@@ -479,14 +493,43 @@ static void _print_accessible (AtkObject *aobject)
   top_tab = gtk_notebook_get_current_page (notebook) + OBJECT;
   _update(top_tab, aobject);
 
-  if (use_magnifier && (ATK_IS_COMPONENT (aobject)))
+  if (use_magnifier)
     {
       gint x, y;
-      gint w, h;
-      atk_component_get_extents (ATK_COMPONENT(aobject),
-                                 &x, &y, &w, &h,
-                                 ATK_XY_SCREEN);
-      _send_to_magnifier (x, y, w, h);
+      gint w=0, h=0;
+      
+      if (ATK_IS_TEXT (aobject))
+        {
+	  gint x0, y0, w0, h0;
+	  gint xN, yN, wN, hN;
+	  gint len;
+	  len = atk_text_get_character_count (ATK_TEXT (aobject));
+	  atk_text_get_character_extents (ATK_TEXT (aobject), 0,
+					  &x0, &y0, &w0, &h0,
+					  ATK_XY_SCREEN);
+          if (len > 0)
+	    {
+	      atk_text_get_character_extents (ATK_TEXT (aobject), len-1,
+					      &xN, &yN, &wN, &hN,
+					      ATK_XY_SCREEN);
+	      x = MIN (x0, xN);
+	      y = MIN (y0, yN);
+	      w = MAX (x0+w0, xN+wN) - x;
+	      h = MAX (y0+h0, yN+hN) - y;
+	    }
+          else
+	    {
+	      x = x0;
+	      y = y0;
+	    }
+        } 
+      else if (ATK_IS_COMPONENT (aobject))
+        {
+	  atk_component_get_extents (ATK_COMPONENT(aobject),
+				     &x, &y, &w, &h,
+				     ATK_XY_SCREEN);
+        }
+      if (w > -1) _send_to_magnifier (x, y, w, h);
     }
 }
 
@@ -519,6 +562,35 @@ _object_is_ours (AtkObject *aobject)
   return FALSE;
 }
 
+static gchar *
+ferret_get_name_from_container (AtkObject *aobject)
+{
+  gchar *s = NULL;
+  gint n = atk_object_get_n_accessible_children (aobject);
+  gint i = 0;
+  
+  while (!s && (i < n))
+    {
+      AtkObject *child;	    
+      child = atk_object_ref_accessible_child (aobject, i);
+      if (ATK_IS_TEXT (child))
+        {
+		gint count = atk_text_get_character_count (ATK_TEXT (child));
+		s = atk_text_get_text (ATK_TEXT (child),
+				       0,
+				       count);
+        }
+      g_object_unref (child);
+      ++i;	    
+    }
+  
+  if (!s)
+    {
+      s = g_strdup ("");
+    }
+  return s;	
+}
+
 static gint
 _print_object (AtkObject *aobject)
 {
@@ -529,6 +601,7 @@ _print_object (AtkObject *aobject)
     G_CONST_RETURN gchar * parent_typename = NULL;
     G_CONST_RETURN gchar * role_name = NULL;
     G_CONST_RETURN gchar * accel_name = NULL;
+    G_CONST_RETURN gchar * text = NULL;
     AtkRole role;
     AtkObject *parent = NULL;
     static AtkObject *prev_aobject = NULL;
@@ -548,7 +621,7 @@ _print_object (AtkObject *aobject)
       index_in_parent = atk_object_get_index_in_parent(aobject);
     n_children = atk_object_get_n_accessible_children(aobject);
     role = atk_object_get_role(aobject);
-    role_name = get_role_name(role);
+    role_name = atk_role_get_name(role);
 
     if (ATK_IS_ACTION (aobject))
       {
@@ -598,8 +671,40 @@ _print_object (AtkObject *aobject)
       {
         if (aobject != prev_aobject)
           {
-            if (!name) name = "no name";
-            _send_to_festival (role_name, name, accel_name);
+	    if (ATK_IS_TEXT (aobject) && !name)
+	      {
+		text = 
+		  atk_text_get_text_at_offset (ATK_TEXT (aobject),
+					       (gint) 0,
+					       ATK_TEXT_BOUNDARY_SENTENCE_END,
+					       (gint *) NULL,
+					       (gint *) NULL);
+		fprintf (stderr, "first sentence: %s\n", text);
+		_send_to_festival (role_name, 
+				   text, "");
+		if (!name) name = "no name";
+	      }
+            else 
+	      { 
+		text = "";
+		if (!name)
+		  {
+		    if (atk_object_get_role (aobject) == ATK_ROLE_TABLE_CELL)
+		      {
+			gchar *cname = ferret_get_name_from_container (aobject);
+			if (cname) name = g_strdup (cname);
+		      }
+		    else if (atk_object_get_role (aobject) == ATK_ROLE_CHECK_BOX)
+		      {
+			name = g_strdup ("check box");
+		      }
+		    else
+		      {
+		        name = "no name";
+		      }
+		  }
+		_send_to_festival (role_name, name, accel_name);
+	      }
           }
       }
 
@@ -702,11 +807,10 @@ _print_relation (AtkObject *aobject)
 
         for (i = 0; i < n_relations; i++)
           {
-            relation = atk_relation_set_get_relation(
-              relation_set, i);
+            relation = atk_relation_set_get_relation (relation_set, i);
 
             relation_type = atk_relation_get_relation_type (relation);
-            relation_name = get_relation_name(relation_type);
+            relation_name = atk_relation_type_get_name (relation_type);
 
             relation_arry = atk_relation_get_target(relation);
 
@@ -763,9 +867,10 @@ _print_state (AtkObject *aobject)
       {
         ATK_STATE_ACTIVE,
         ATK_STATE_CHECKED,
-        ATK_STATE_COLLAPSED,
+        ATK_STATE_EXPANDED,
         ATK_STATE_EXPANDABLE,
         ATK_STATE_SELECTED,
+        ATK_STATE_SHOWING,
         ATK_STATE_VISIBLE
       };
 
@@ -1060,7 +1165,7 @@ _print_table (AtkTable *aobject)
 static gint
 _print_text (AtkText *aobject)
 {
-    gchar *output_str, *text_val;
+    gchar *output_str, *text_val, *text_val_escaped;
     gint n_chars, caret_offset;
     gint start_offset, end_offset;
     gint group_num;
@@ -1081,12 +1186,14 @@ _print_text (AtkText *aobject)
     * Pass through g_strescape so that non-ASCII chars are made
     * print-able.
     */
-    text_val = g_strescape(atk_text_get_text (aobject, 0, n_chars), NULL);
-
-    if (output_str)
+    text_val = atk_text_get_text (aobject, 0, n_chars);
+    if (text_val)
       {
-        _print_key_value (tab_n, group_num, "Text", (gpointer)text_val,
+        text_val_escaped = g_strescape(text_val, NULL);
+        _print_key_value (tab_n, group_num, "Text", (gpointer)text_val_escaped,
           VALUE_TEXT);
+        g_free (text_val);
+        g_free (text_val_escaped);
       }
     else
       {
@@ -1099,15 +1206,16 @@ _print_text (AtkText *aobject)
       VALUE_STRING);
     g_free(output_str);
 
-    output_str = atk_text_get_text_at_offset (aobject, caret_offset,
-                                              ATK_TEXT_BOUNDARY_CHAR,
-                                              &start_offset, &end_offset);
-
-    if (output_str)
+    text_val = atk_text_get_text_at_offset (aobject, caret_offset,
+                                            ATK_TEXT_BOUNDARY_CHAR,
+                                            &start_offset, &end_offset);
+    if (text_val)
       {
+        text_val_escaped = g_strescape(text_val, NULL);
         _print_key_value(tab_n, group_num, "Current Character",
-          (gpointer)output_str, VALUE_STRING);
-        g_free(output_str);
+          (gpointer)text_val_escaped, VALUE_STRING);
+        g_free (text_val);
+        g_free (text_val_escaped);
       }
     else
       {
@@ -1135,37 +1243,56 @@ _print_text (AtkText *aobject)
         g_free(output_str);
       }
 
-    output_str = atk_text_get_text_at_offset (aobject, caret_offset,
-                                              ATK_TEXT_BOUNDARY_WORD_START,
-                                              &start_offset, &end_offset);
-
-    _print_key_value(tab_n, group_num, "Current Word", (gpointer)output_str,
-                     VALUE_STRING);
-    if (output_str)
+    text_val = atk_text_get_text_at_offset (aobject, caret_offset,
+                                            ATK_TEXT_BOUNDARY_WORD_START,
+                                            &start_offset, &end_offset);
+    if (text_val)
       {
-        g_free(output_str);
+        text_val_escaped = g_strescape(text_val, NULL);
+        _print_key_value(tab_n, group_num, "Current Word",
+          (gpointer)text_val_escaped, VALUE_STRING);
+        g_free (text_val);
+        g_free (text_val_escaped);
+      }
+    else
+      {
+        _print_key_value(tab_n, group_num, "Current Word", "none",
+          VALUE_STRING);
       }
 
-    output_str = atk_text_get_text_at_offset (aobject, caret_offset,
-                                              ATK_TEXT_BOUNDARY_LINE_START,
-                                              &start_offset, &end_offset);
-    _print_key_value(tab_n, group_num, "Current Line", (gpointer)output_str,
-                         VALUE_TEXT);
-    if (output_str)
+    text_val = atk_text_get_text_at_offset (aobject, caret_offset,
+                                            ATK_TEXT_BOUNDARY_LINE_START,
+                                            &start_offset, &end_offset);
+    if (text_val)
       {
-        g_free(output_str);
+        text_val_escaped = g_strescape(text_val, NULL);
+        _print_key_value(tab_n, group_num, "Current Line",
+          (gpointer)text_val_escaped, VALUE_STRING);
+        g_free (text_val);
+        g_free (text_val_escaped);
+      }
+    else
+      {
+        _print_key_value(tab_n, group_num, "Current Line", "none",
+          VALUE_STRING);
       }
 
-    output_str = atk_text_get_text_at_offset (aobject, caret_offset,
+    text_val = atk_text_get_text_at_offset (aobject, caret_offset,
                                               ATK_TEXT_BOUNDARY_SENTENCE_START,
                                               &start_offset, &end_offset);
-    _print_key_value(tab_n, group_num, "Current Sentence", (gpointer)output_str,
-      VALUE_TEXT);
-    if (output_str)
+    if (text_val)
       {
-        g_free(output_str);
+        text_val_escaped = g_strescape(text_val, NULL);
+        _print_key_value(tab_n, group_num, "Current Sentence",
+          (gpointer)text_val_escaped, VALUE_STRING);
+        g_free (text_val);
+        g_free (text_val_escaped);
+      }
+    else
+      {
+        _print_key_value(tab_n, group_num, "Current Line", "none",
+          VALUE_STRING);
       }
-
     return(group_num);
 }
 
@@ -1192,7 +1319,7 @@ _print_text_attributes (AtkText *aobject)
     start_offset = 0;
     end_offset = 0;
 
-    attribute_set = atk_text_ref_run_attributes(aobject, attribute_offset,
+    attribute_set = atk_text_get_run_attributes(aobject, attribute_offset,
           &start_offset, &end_offset);
 
     label_str = g_strdup_printf("Attribute run start");
@@ -1295,7 +1422,7 @@ _print_value_type(gint group_num, gchar *type, GValue *value)
 static void
 _create_event_watcher ()
 {
-    atk_add_focus_tracker (_print_accessible);
+    focus_tracker_id = atk_add_focus_tracker (_print_accessible);
 
     if (track_mouse)
       {
@@ -1347,7 +1474,7 @@ _button_watcher (GSignalInvocationHint *ihint,
     object = g_value_get_object (param_values + 0);
 
     widget = GTK_WIDGET (object);
-    if (GTK_IS_TREE_VIEW (widget) || GTK_IS_CLIST (widget))
+    if (GTK_IS_CONTAINER (widget))
     {
       if (G_VALUE_HOLDS_BOXED (param_values + 1))
         {
@@ -1377,9 +1504,15 @@ _button_watcher (GSignalInvocationHint *ihint,
               g_object_unref (child);
             }
           else
-               g_print ("No child at position %d %d\n", 
-                        x,
-                        y);
+            {
+              if (!GTK_IS_MENU_ITEM (widget))
+                {
+                  g_print ("No child at position %d %d for %s\n", 
+                           x,
+                           y,
+                           g_type_name (G_OBJECT_TYPE (widget)));
+                }
+            }
         }
     }
 
@@ -1533,12 +1666,14 @@ _create_window ()
 
         _add_menu(&menu, &menuitem_trackmouse, "Track Mouse", track_mouse,
            GTK_SIGNAL_FUNC(_toggle_trackmouse));
+        _add_menu(&menu, &menuitem_trackfocus, "Track Focus", track_focus,
+           GTK_SIGNAL_FUNC(_toggle_trackfocus));
         _add_menu(&menu, &menuitem_magnifier, "Magnifier", use_magnifier,
            GTK_SIGNAL_FUNC(_toggle_magnifier));
         _add_menu(&menu, &menuitem_festival, "Festival", use_festival,
            GTK_SIGNAL_FUNC(_toggle_festival));
         _add_menu(&menu, &menuitem_festival_terse, "Festival Terse",
-          (say_role == FALSE && say_accel == FALSE),
+          (!say_role && !say_accel),
           GTK_SIGNAL_FUNC(_toggle_festival_terse));
         _add_menu(&menu, &menuitem_terminal, "Terminal Output", display_ascii,
            GTK_SIGNAL_FUNC(_toggle_terminal));
@@ -1570,25 +1705,25 @@ _add_menu(GtkWidget ** menu, GtkWidget ** menuitem, gchar * name,
 int
 gtk_module_init(gint argc, char* argv[])
 {
-    if (getenv ("FERRET_ASCII"))
+    if (g_getenv ("FERRET_ASCII"))
        display_ascii = TRUE;
 
-    if (getenv ("FERRET_NOSIGNALS"))
+    if (g_getenv ("FERRET_NOSIGNALS"))
        no_signals = TRUE;
 
     if (display_ascii)
        g_print("GTK ferret Module loaded\n");
 
-    if (getenv("FERRET_MAGNIFIER"))
+    if (g_getenv("FERRET_MAGNIFIER"))
         use_magnifier = TRUE;
 
-    if (getenv ("FERRET_FESTIVAL"))
+    if (g_getenv ("FERRET_FESTIVAL"))
         use_festival = TRUE;
 
-    if (getenv ("FERRET_MOUSETRACK"))
+    if (g_getenv ("FERRET_MOUSETRACK"))
         track_mouse = TRUE;
 
-    if (getenv ("FERRET_TERSE"))
+    if (g_getenv ("FERRET_TERSE"))
       {
         say_role = FALSE;
         say_accel = FALSE;
@@ -1691,7 +1826,7 @@ _get_group(TabInfo *tab, GroupId group_id, const gchar *groupname)
           }
       }
 
-   if (found == FALSE)
+   if (!found)
      {
        /* build a new one */
        group = (GroupInfo *)g_new0(GroupInfo, 1);
@@ -1799,14 +1934,14 @@ _get_name_value(GroupInfo *group, const gchar *label,
     for (nv_list = group->name_value; nv_list; nv_list = nv_list->next)
       {
         name_value = (NameValue *) nv_list->data;
-        if (name_value->active == FALSE)
+        if (!name_value->active)
           {
             found = TRUE;
             break;
           }
       }
 
-    if (found == FALSE)
+    if (!found)
       {
         name_value = (NameValue *)g_new0(NameValue, 1);
         name_value->column1 = GTK_HBOX(gtk_hbox_new(FALSE, 10));
@@ -1815,7 +1950,7 @@ _get_name_value(GroupInfo *group, const gchar *label,
         name_value->label = GTK_LABEL(gtk_label_new(label));
         name_value->string = gtk_label_new (NULL);
         name_value->boolean = gtk_check_button_new ();
-        name_value->text = gtk_entry_new ();
+        name_value->text = gtk_entry_new_with_max_length (1000);
         name_value->button = GTK_BUTTON(gtk_button_new ());
 
         gtk_box_pack_end(GTK_BOX(name_value->column1),
@@ -1931,14 +2066,15 @@ _print_key_value(TabNumber tab_n, gint group_number,
     {
       if (type == VALUE_BOOLEAN)
         {
-          if (*((gboolean *)value) == TRUE)
+          if (*((gboolean *)value))
               g_print("\t%-30s\tTRUE\n", label);
           else
               g_print("\t%-30s\tFALSE\n", label);
         }
       else
         {
-          g_print("\t%-30s\t%s\n", label, (gchar *)value);
+          g_print("\t%-30s\t%s\n", label, 
+                  value ? (gchar *)value : "NULL");
         }
     }
 
@@ -1969,6 +2105,7 @@ _finished_group(TabNumber tab_no, gint group_number)
 
 static gulong child_added_id = 0;
 static gulong child_removed_id = 0;
+static gulong state_change_id = 0;
 
 static gulong text_caret_handler_id = 0;
 static gulong text_inserted_id = 0;
@@ -1994,6 +2131,8 @@ _update_handlers(AtkObject *obj)
            g_signal_handler_disconnect(last_object, child_added_id);
         if (child_removed_id != 0)
            g_signal_handler_disconnect(last_object, child_removed_id);
+        if (state_change_id != 0)
+           g_signal_handler_disconnect(last_object, state_change_id);
 
         if (text_caret_handler_id != 0)
            g_signal_handler_disconnect(last_object, text_caret_handler_id);
@@ -2035,7 +2174,7 @@ _update_handlers(AtkObject *obj)
     table_column_reordered_id = 0;
     property_id = 0;
 
-    if (G_TYPE_CHECK_INSTANCE(obj) == FALSE)
+    if (!G_TYPE_CHECK_INSTANCE(obj))
         return;
 
     g_object_ref(obj);
@@ -2054,6 +2193,11 @@ _update_handlers(AtkObject *obj)
                 "children_changed::remove",
                 g_cclosure_new (G_CALLBACK (_notify_object_child_removed),
                 NULL, NULL), FALSE);
+
+         state_change_id = g_signal_connect_closure (obj,
+                "state_change",
+                g_cclosure_new (G_CALLBACK (_notify_object_state_change),
+                NULL, NULL), FALSE);
       }
 
     if (ATK_IS_TEXT(obj))
@@ -2110,20 +2254,22 @@ _update_handlers(AtkObject *obj)
 /* Text signals */
 
 static void
-_notify_text_insert_handler (GObject *obj, int start_offset, int end_offset)
+_notify_text_insert_handler (GObject *obj, int position, int offset)
 {
-    gchar *output_str = g_strdup_printf("position %d, length %d!",
-      start_offset, end_offset);
+    gchar *text = atk_text_get_text (ATK_TEXT (obj), position, position + offset);
+    gchar *output_str = g_strdup_printf("position %d, length %d text: %s",
+      position, offset,  text ? text: "<NULL>");
     _print_signal(ATK_OBJECT(obj), FERRET_SIGNAL_TEXT,
       "Text Insert", output_str);
     g_free(output_str);
 }
 
 static void
-_notify_text_delete_handler (GObject *obj, int start_offset, int end_offset)
+_notify_text_delete_handler (GObject *obj, int position, int offset)
 {
-    gchar *output_str = g_strdup_printf("position %d, length %d!",
-      start_offset, end_offset);
+    gchar *text = atk_text_get_text (ATK_TEXT (obj), position, position + offset);
+    gchar *output_str = g_strdup_printf("position %d, length %d text: %s",
+      position, offset,  text ? text: "<NULL>");
     _print_signal(ATK_OBJECT(obj), FERRET_SIGNAL_TEXT,
       "Text Delete", output_str);
     g_free(output_str);
@@ -2222,6 +2368,17 @@ _notify_object_child_removed (GObject *obj, gint index,
     g_free(output_str);
 }
 
+static void 
+_notify_object_state_change (GObject *obj, gchar *name, gboolean set)
+{
+    gchar *output_str = g_strdup_printf("name %s %s set", 
+                        name, set ? "is" : "was");
+    _print_signal(ATK_OBJECT(obj), FERRET_SIGNAL_OBJECT,
+      "State Change", output_str);
+    g_free(output_str);
+}
+
+
 /* Function to print signals */
 
 static void
@@ -2247,7 +2404,7 @@ _print_signal(AtkObject *aobject, FerretSignalType type,
           {
             _speak_caret_event (aobject);
          }
-        else
+        else if (type == FERRET_SIGNAL_TEXT)
           {
             last_caret_offset = atk_text_get_caret_offset (ATK_TEXT (aobject));
           }
@@ -2276,7 +2433,7 @@ _print_signal(AtkObject *aobject, FerretSignalType type,
 /* Update functions */
 
 static void
-_update(TabNumber top_tab, AtkObject *aobject)
+_update (TabNumber top_tab, AtkObject *aobject)
 {
     gint group_num;
 
@@ -2379,16 +2536,12 @@ static void _property_change_handler (AtkObject *obj,
 
     if (top_tab == OBJECT &&
        ((strcmp (values->property_name, "accessible-name") == 0) ||
-        (strcmp (values->property_name, "accessible-state") == 0) ||
         (strcmp (values->property_name, "accessible-description") == 0) ||
-        (strcmp (values->property_name, "accessible-child") == 0) ||
         (strcmp (values->property_name, "accessible-parent") == 0) ||
-        (strcmp (values->property_name, "accessible-caret") == 0) ||
-        (strcmp (values->property_name, "accessible-selection") == 0) ||
-        (strcmp (values->property_name, "accessible-text") == 0) ||
         (strcmp (values->property_name, "accessible-value") == 0) ||
-        (strcmp (values->property_name, "accessible-visible-data") == 0) ||
         (strcmp (values->property_name, "accessible-role") == 0) ||
+        (strcmp (values->property_name, "accessible-component-layout") == 0) ||
+        (strcmp (values->property_name, "accessible-component-mdi-zorder") == 0) ||
         (strcmp (values->property_name, "accessible-table-caption") == 0) ||
         (strcmp (values->property_name,
                  "accessible-table-column-description") == 0) ||
@@ -2398,8 +2551,7 @@ static void _property_change_handler (AtkObject *obj,
                  "accessible-table-row-description") == 0) ||
         (strcmp (values->property_name,
                  "accessible-table-row-header") == 0) ||
-        (strcmp (values->property_name, "accessible-table-summary") == 0) ||
-        (strcmp (values->property_name, "accessible-model") == 0)))
+        (strcmp (values->property_name, "accessible-table-summary") == 0)))
       {
         if (display_ascii)
             g_print("Updating tab\n");
@@ -2500,3 +2652,19 @@ void _toggle_trackmouse(GtkCheckMenuItem *checkmenuitem,
          }
      }
 }
+
+void _toggle_trackfocus(GtkCheckMenuItem *checkmenuitem,
+  gpointer user_data)
+{
+   if (checkmenuitem->active)
+     {
+       track_focus = TRUE;
+       focus_tracker_id = atk_add_focus_tracker (_print_accessible);
+     }
+   else
+     { 
+       g_print ("No longer tracking focus.\n");
+       track_focus = FALSE;
+       atk_remove_focus_tracker (focus_tracker_id);
+     }
+}
